package org.amse.im.practice.surface.impl;

import org.amse.im.practice.exceptions.*;
import org.amse.im.practice.formula.*;
import org.amse.im.practice.formula.impl.*;
import org.amse.im.practice.surface.*;


public class Surface implements ISurface{
	private int myCountT, myCountU;
	private Point3D[][] myGridOfPoint;
	private IFormula myFormula; 
	private ConstantCountExpression myCountUExpression, myCountTExpression;
	private boolean isValid = true;
	private String myStringCountU, myStringCountT;
	
	public Surface(IFormula formula, String stringCountU, String stringCountT){
		myFormula = formula;
		myStringCountU = stringCountU;
		myStringCountT = stringCountT;
		if (!myFormula.isValid()) {
			isValid = false;
		} else {
			try {
				myCountTExpression = new ConstantCountExpression(stringCountT);
				myCountUExpression = new ConstantCountExpression(stringCountU);
		
				myCountU = (int)myCountUExpression.value();
				myCountT = (int)myCountTExpression.value();	
				myGridOfPoint = new Point3D[myCountU][myCountT];
				init();
			} catch(MyException e) {
				isValid = false;
				System.out.println(e);
			}
		}	
	}
	
	public IFormula getFormula() {
		return myFormula;
	}
	
	public String getStringCountU() {
		return myStringCountU;
	}
	
	public String getStringCountT() {
		return myStringCountT;
	}
	
	private void init() {
		for(int i = 0; i < myCountU; i++){
			
			double u = (myFormula.getEndU() - myFormula.getStartU()) / (myCountU - 1) * i
				+ myFormula.getStartU();
			
			for(int j = 0; j < myCountT; j++){	
				double t = (myFormula.getEndT() - myFormula.getStartT()) / (myCountT - 1) * j 
							+ myFormula.getStartT();
				myGridOfPoint[i][j] = myFormula.evaluate(u, t);
			}
		}
	}
	
	public boolean isValid() {
		return isValid;
	}
	
	public int getTCount() {
		return myCountT;
	}
	
	public int getUCount() {
		return myCountU;
	}
	
	public Point3D getValue(int i, int j) {
		return myGridOfPoint[i][j];
	}

}
