package org.amse.im.practice.test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.*;

import org.amse.im.practice.formula.syntaxTree.node.*;
import org.amse.im.practice.formula.syntaxTree.tokenType.*;

import junit.framework.TestCase;

public class BuildTreeTest extends TestCase {
	String myAllChars;
	
	private boolean isAnswerCorrect(String actualFile, String expectedFile ){
		boolean isCorrect = true;
		Reader readerActualFile = null;
		Reader readerExpectedFile = null;
		
		try{
			readerActualFile = new BufferedReader(new FileReader(actualFile));
			readerExpectedFile = new BufferedReader(new FileReader("org\\tests\\" + expectedFile));
		} catch(IOException e){
			System.out.println("error in opening");
		}
		
		int c2 = 0;
		try{
			try{
				int c1 = readerExpectedFile.read();
				c2 = readerActualFile.read();
				myLabel:
				while(c1 != -1){
					if (c2 != c1){
						isCorrect =  false;
						break myLabel;
					}
					c1 = readerExpectedFile.read();
					c2 = readerActualFile.read();
				}
			}finally{
				readerActualFile.close();
				readerExpectedFile.close();
			}	
		}catch(IOException e){
			System.out.println("error in reading");
		}
		
		return (c2 == -1 ? isCorrect : false);
	}
	
	public String createActualFile(String string, String fileName){
		INode head = TreeBuilder.buildSyntaxTree(string);		
		Writer writer = null;
		try{
			ArrayList<IToken> list = TreeBuilder.tokenize(string);
			StringBuffer sb = new StringBuffer();
			for (IToken token : list) {
				sb.append(token.toString() + " ");
			}
			writer = new BufferedWriter(new FileWriter(fileName));
			writer.write(sb.toString() + " end" + "\n");
			print(head, 0, writer);
		} catch(IOException e){
			System.out.println("error in opening");
		}
		try{
			writer.close();
		}catch (IOException e){
			System.out.println("error in closing");
		}
		
		return fileName;
	}
	
	private String spacePrinter(int spaceCounter){
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < spaceCounter; i++){
			sb.append(' ');
		}
		return sb.toString();
	}
	
	private void print(INode node, int spaceCounter, Writer writer){
		try{
			writer.write(spacePrinter(spaceCounter) + node.toString() + "\n");
		} catch(IOException e){
			System.out.println("error in writing");
		}
		for (INode child : node.children()) {
			print(child, spaceCounter + 2, writer);
		}
	}
		
	public void test1(){
		String allChars = "((-22-5)*(1+11*1))";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual1.txt"), "outputExpected1.txt"));
		new File("outputActual1.txt").delete();
	}
	
	public void test2(){
		String allChars = " ( (  22 + 5)*(1+11*1) +  (5  -   2*5))  ";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual2.txt"), "outputExpected2.txt"));
		new File("outputActual2.txt").delete();	
	}
	
	public void test3(){
		String allChars = "((-22-5)*(1+10*1))+(-cos(3) + t)";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual3.txt"), "outputExpected3.txt"));
		new File("outputActual3.txt").delete();
	}
	
	public void test4(){
		String allChars = "-5+sin(u)-u";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual4.txt"), "outputExpected4.txt"));
		new File("outputActual4.txt").delete();
	}
	
	public void test5(){
		String allChars = "(5*(1+1))";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual5.txt"), "outputExpected5.txt"));
		new File("outputActual5.txt").delete();
	}
	
	public void test6(){
		String allChars = "4 + t";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual6.txt"), "outputExpected6.txt"));
		new File("outputActual6.txt").delete();
	}
	
	public void test7(){
		String allChars = "sin(t)";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual7.txt"), "outputExpected7.txt"));
		new File("outputActual7.txt").delete();
	}
	
	public void test8(){
		String allChars = "sin(t * cos(u + t))";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual8.txt"), "outputExpected8.txt"));
		new File("outputActual8.txt").delete();
	}
	
	public void test9(){
		String allChars = "sin(t * cos(u + t)) - 8 - 4 + 3*2";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual9.txt"), "outputExpected9.txt"));
		new File("outputActual9.txt").delete();
	}
	
	public void test10(){
		String allChars = "cos(u) * sin(t) * (100)";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual10.txt"), "outputExpected10.txt"));
		new File("outputActual10.txt").delete();	
	}
	
	public void test11(){
		String allChars = "5";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual11.txt"), "outputExpected11.txt"));
		new File("outputActual11.txt").delete();	
	}
	
	public void test12(){
		String allChars = "-100";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual12.txt"), "outputExpected12.txt"));
		new File("outputActual12.txt").delete();	
	}
	
	public void test13(){
		String allChars = "u - t";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual13.txt"), "outputExpected13.txt"));
		new File("outputActual13.txt").delete();	
	}
	
	public void test14(){
		String allChars = "u - t + 2 + 4";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual14.txt"), "outputExpected14.txt"));
		new File("outputActual14.txt").delete();	
	}
	
	public void test15(){
		String allChars = "13 * 3426";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual15.txt"), "outputExpected15.txt"));
		new File("outputActual15.txt").delete();	
	}
	
	public void test16(){
		String allChars = "sin(13) * 26";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual16.txt"), "outputExpected16.txt"));
		new File("outputActual16.txt").delete();	
	}
	
	public void test17(){
		String allChars = "-pi";
		assertTrue(isAnswerCorrect(createActualFile(allChars, "outputActual17.txt"), "outputExpected17.txt"));
		new File("outputActual17.txt").delete();	
	}
	
	
	
	
}











