package org.amse.im.practice.test;

import org.amse.im.practice.formula.syntaxTree.node.*;
import junit.framework.*;

public class EvaluateTest extends TestCase {
	String myAllChars;
	
	private double evaluate(String string, double u, double t){
		return TreeBuilder.buildSyntaxTree(myAllChars).evaluate(u, t);
	}
	
	public void test1(){
		myAllChars = " 100 * sin(-u) * cos ( t)  ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (100*Math.cos(t)*Math.sin(-u)), 0.05);		
	}
	
	public void test2(){
		myAllChars = " 100 * sin(-5*(-2-3-4*(u+1)))  ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (100*Math.sin(-5*(-2-3-4*(u+1)))), 0.05);	
	}
	
	public void test3(){
		myAllChars = " 100 * sin(-u)";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (100 * Math.sin(-u)), 0.05);
	}
	
	public void test4(){
		myAllChars = " 5 ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (5.0), 0.05);
	}
	
	public void test5(){
		myAllChars = " u ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (u), 0.05);
	}
	
	public void test6(){
		myAllChars = " t + 3 ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (t + 3.0), 0.05);
	}
	public void test7(){
		myAllChars = " u * t ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (u * t), 0.05);
	}
	
	public void test8(){
		myAllChars = " -5 ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (-5.0), 0.05);
	}
	
	public void test9(){
		myAllChars = " -u - 4 ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (-u - 4.0), 0.05);
	}
	
	public void test10(){
		myAllChars = " 5/ (-t) ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (5.0 / (-t)), 0.05);
	}
	
	public void test11(){
		myAllChars = " cos(3) ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (Math.cos(3.0)), 0.05);
	}
	
	public void test12(){
		myAllChars = "- cos(sin(-u-t)) ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (-Math.cos(Math.sin(-u-t))), 0.05);
	}
	
	public void test13(){
		myAllChars = " 5 + u + 1 ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (5.0 + u + 1), 0.05);
	}
	public void test14(){
		myAllChars = " 5 + u -2- 4 -6 +7*t ";
		double u = 2;
		double t = 4;
		assertEquals(evaluate(myAllChars, u, t), (5.0 + u -2.0- 4.0 -6.0 +7.0*t), 0.05);
	}
	
	public void test15(){
		myAllChars = " 2 * pi ";
		double u = 3;
		double t = 45;

		assertEquals(evaluate(myAllChars, u, t), (Math.PI * 2), 0.05);
	}
	
	public void test16(){
		myAllChars = " - pi ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (-Math.PI), 0.05);
	}
	
	public void test17(){
		myAllChars = " sqrt(4) ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (Math.sqrt(4)), 0.05);
	}
	
	public void test18(){
		myAllChars = " sqrt(u) ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (Math.sqrt(u)), 0.05);
	}
	
	public void test19(){
		myAllChars = " sqrt(u*sin(3)) ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (Math.sqrt(u*Math.sin(3))), 0.05);
	}
	
	public void test20(){
		myAllChars = " 1/2 ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), ((double)1/2), 0.05);
	}
	
	public void test21(){
		myAllChars = " (u + t)/2 ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), ((u + t)/2), 0.05);
	}
	
	public void test22(){
		myAllChars = " sin(u / 2) / t ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (Math.sin(u / 2)/t), 0.05);
	}
	public void test23(){
		myAllChars = "100*sin(u)*cos(t) ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (100* Math.sin(u)* Math.cos(t)), 0.05);
	}
	
	public void test24(){
		myAllChars = "100*sin(u)*sin(t) ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (100* Math.sin(u)* Math.sin(t)), 0.05);
	}
	
	public void test25(){
		myAllChars = "100*cos(u) ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (100* Math.cos(u)), 0.05);
	}
	
	public void test26(){
		myAllChars = "100.0 ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (100), 0.05);
	}
	
	public void test27(){
		myAllChars = "0.03 ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (0.03), 0.05);
	}
	
	public void test28(){
		myAllChars = "1345.0231 ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (1345.0231), 0.05);
	}
	
	public void test29(){
		myAllChars = "-1.1 ";
		double u = 3;
		double t = 45;
		assertEquals(evaluate(myAllChars, u, t), (-1.1), 0.05);
	}
	
	
}














