package org.amse.im.practice.test;

import java.util.*;

import org.amse.im.practice.formula.syntaxTree.node.*;
import org.amse.im.practice.formula.syntaxTree.tokenType.*;

import junit.framework.TestCase;

public class TokenizeTest extends TestCase {
	
	private String createActualString(String string){
		List<IToken> list = TreeBuilder.tokenize(string);
		StringBuffer sb = new StringBuffer();
		for (IToken token : list) {
			sb.append(token + " ");
		}
		sb.append("end");
		return sb.toString();	
	}
	
	public void test1(){
		String actual = createActualString("( ( -1 * 22 - 5 ) * ( 1 + 11 * 1 ) )");
		String expected = "( ( - 1 * 22 - 5 ) * ( 1 + 11 * 1 ) ) end";
		assertEquals(expected, actual);
	}
	
	public void test2(){
		String actual = createActualString(" ( (  22 + 5)*(1+11*1) +  (5  -   2*5))  ");
		String expected = "( ( 22 + 5 ) * ( 1 + 11 * 1 ) + ( 5 - 2 * 5 ) ) end";
		assertEquals(expected, actual);
	}
	
	public void test3(){
		String actual = createActualString("((-22-5)*(1+10*1))+(-cos(3) + t)");
		String expected = "( ( - 22 - 5 ) * ( 1 + 10 * 1 ) ) + ( - cos ( 3 ) + t ) end";
		assertEquals(expected, actual);
	}
	
	public void test4(){
		String actual = createActualString("-5 + sin ( u ) - u ");
		String expected = "- 5 + sin ( u ) - u end";
		assertEquals(expected, actual);
	}
	
	public void test5(){
		String actual = createActualString("-5 ");
		String expected = "- 5 end";
		assertEquals(expected, actual);
	}
	
	public void test6(){
		String actual = createActualString("-5 + u");
		String expected = "- 5 + u end";
		assertEquals(expected, actual);
	}
	
	public void test7(){
		String actual = createActualString("100");
		String expected = "100 end";
		assertEquals(expected, actual);
	}
	
	public void test8(){
		String actual = createActualString("(t)");
		String expected = "( t ) end";
		assertEquals(expected, actual);
	}
	
	public void test9(){
		String actual = createActualString("t * u");
		String expected = "t * u end";
		assertEquals(expected, actual);
	}
	
	public void test10(){
		String actual = createActualString("(5+ 4 +(u)-t) ");
		String expected = "( 5 + 4 + ( u ) - t ) end";
		assertEquals(expected, actual);
	}
	
	public void test11(){
		String actual = createActualString("() ");
		String expected = "( ) end";
		assertEquals(expected, actual);
	}
	
	public void test12(){
		String actual = createActualString("cos(9)");
		String expected = "cos ( 9 ) end";
		assertEquals(expected, actual);
	}
	
	public void test13(){
		String actual = createActualString("sin ( cos(u/t) ) - u ");
		String expected = "sin ( cos ( u / t ) ) - u end";
		assertEquals(expected, actual);
	}
	
	public void test14(){
		String actual = createActualString("- u ");
		String expected = "- u end";
		assertEquals(expected, actual);
	}
	
	public void test15(){
		String actual = createActualString("sqrt( u ) ");
		String expected = "sqrt ( u ) end";
		assertEquals(expected, actual);
	}
	
	public void test16(){
		String actual = createActualString("sqrt( 5* (-u-t) ) ");
		String expected = "sqrt ( 5 * ( - u - t ) ) end";
		assertEquals(expected, actual);
	}
	
//	check on valid of string 
	public void test17(){
		String actual = createActualString(" ");
		String expected = "end";
		assertEquals(expected, actual);
	}
	
	public void test18(){
		String actual = createActualString("55 + 3");
		String expected = "55 + 3 end";
		assertEquals(expected, actual);
	}
	
	public void test19(){
		String actual = createActualString("sqrt( 5* (-u-t) ) ");
		String expected = "sqrt ( 5 * ( - u - t ) ) end";
		assertEquals(expected, actual);
	}
	
	public void test20(){
		String actual = createActualString(" ( ( ) )  +  3333");
		String expected = "( ( ) ) + 3333 end";
		assertEquals(expected, actual);
	}
}






