package org.amse.im.practice.view;

import org.amse.im.practice.exceptions.*;
import org.amse.im.practice.formula.*;
import org.amse.im.practice.formula.impl.*;
import org.amse.im.practice.io.*;
import org.amse.im.practice.surface.*;
import org.amse.im.practice.surface.impl.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import javax.swing.filechooser.FileFilter;


public class View extends JFrame{
	private ISurface mySurface;
	private IFormula myFormula;
	private FormulaTextField myTextXFormula, myTextYFormula, myTextZFormula;
	private FormulaTextField myTextStartU, myTextEndU, myTextStartT, myTextEndT;
	private FormulaTextField myTextCountU, myTextCountT;
	private JButton myDrawButton;
	private JPanel myPanel;
	private boolean myIsSaved = true;
	private int myWrongFieldCounter = 0;
	
	public View(){
		super("3dGeometry");
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new ExitListener());
		myPanel = new JPanel();
		myPanel.setLayout(new BorderLayout());
		myPanel.add(createFormulaPanel(), BorderLayout.WEST);
		getContentPane().add(myPanel);
		
		JMenuBar menuBar = new JMenuBar();
		menuBar.add(createFileMenu());
		setJMenuBar(menuBar);
		init();
		setVisible(true);
		pack();
	}
	
	private JMenu createFileMenu(){
		JMenu file = new JMenu("File");
		JMenuItem open = new JMenuItem("Open");
		open.addActionListener(new OpenListener());
		
		JMenuItem save = new JMenuItem("Save");
		save.addActionListener(new SaveListener());
		JMenuItem exit = new JMenuItem("Exit");
		exit.addActionListener(new AbstractAction(){
			public void actionPerformed(ActionEvent e){
				if (!myIsSaved) {
					int result = JOptionPane.showConfirmDialog(null, "Do you want to save?");
					if (result == JOptionPane.YES_OPTION) {
						saveAction();
//					getToolkit().getSystemEventQueue().postEvent(		
//					new ActionEvent(save, ActionEvent.ACTION_PERFORMED, null));
					}
					if (result != JOptionPane.CANCEL_OPTION) {
						System.exit(0);
					}
				} else {
					System.exit(0);
				}
			}
		});
		file.add(open);
		file.add(save);
		file.addSeparator();
		file.add(exit);
		
		return file;
	}
	
	private class ExitListener extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			if (!myIsSaved) {
				int result = JOptionPane.showConfirmDialog(null, "Do you want to save?");
				if (result == JOptionPane.YES_OPTION) {
					saveAction();
//				getToolkit().getSystemEventQueue().postEvent(		
//				new ActionEvent(save, ActionEvent.ACTION_PERFORMED, null));
				}
				if (result != JOptionPane.CANCEL_OPTION) {
					System.exit(0);
				}
			} else {
				System.exit(0);
			}
		}
	}
	
	private class OpenListener extends AbstractAction {
		public void actionPerformed(ActionEvent e) {
			JFileChooser fileChooser = new JFileChooser(); 
			fileChooser.setDialogTitle("Open");
			fileChooser.addChoosableFileFilter(new MyFilesFilter());
			fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
			int result = fileChooser.showOpenDialog(View.this);
			if (result == JFileChooser.APPROVE_OPTION) {
				try {
					FileInputStream fileInputStream = new FileInputStream(fileChooser.getSelectedFile());
					XMLReader1 xmlReader1 = new XMLReader1(fileInputStream);
					myFormula = xmlReader1.getFormula();
					mySurface = xmlReader1.getSurface();
					initAfterReading();
				} catch(Exception exception) {
					System.out.println("OpenListener " + exception);
				}
			}
		}
	}
	
	private class SaveListener extends AbstractAction{
		public void actionPerformed(ActionEvent e){
			myIsSaved = true;
			saveAction();
		}
	}
	
	private void saveAction() {
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogTitle("Save as");
		fileChooser.addChoosableFileFilter(new MyFilesFilter());
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		int res = fileChooser.showOpenDialog(View.this);
		if (res == JFileChooser.APPROVE_OPTION) {
			try {
				myFormula = new Formula(myTextXFormula.getText(),myTextYFormula.getText(),
						myTextZFormula.getText(), myTextStartT.getText(), myTextEndT.getText(),
						myTextStartU.getText(), myTextEndU.getText());
				mySurface = new Surface(myFormula, myTextCountU.getText(), myTextCountT.getText());
				File file = fileChooser.getSelectedFile();
				if (!file.getName().endsWith("srf")) {
					file = new File(file.getAbsolutePath() + ".srf");
				}
				FileOutputStream fileOutputStream = new FileOutputStream(file);
				new XMLWriter().write(mySurface, fileOutputStream);
			} catch(Exception exception) {
				exception.printStackTrace();
			}
		}	
	}
	
	private class MyFilesFilter extends FileFilter {
		public boolean accept(File file) {
			if (file.isDirectory()) {
				return true;
			}
			return file.getName().endsWith("srf");
		}
		public String getDescription() {
			return "XML files(*.srf)";
		}
	}
	
	
	private FormulaTextField createVariablePanel(String string, JPanel main, String content, int fieldSize, 
										boolean constant, boolean isCount){
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		JLabel name = new JLabel(string + " = ");
		panel.add(name);
		panel.add(Box.createVerticalStrut(12));
		FormulaTextField textField = constant ?
			isCount ? 
				new ConstantCountFormulaTextField(content, fieldSize, this) :
				new ConstantFormulaTextField(content, fieldSize, this) :	
			new NonConstantFormulaTextField(content, fieldSize, this);
		textField.addCaretListener(new MyCaretListener(textField));
		setTextFieldSize(textField);
		panel.add(textField);
		main.add(panel);

		return textField;
	}
	
	private void setTextFieldSize(JTextField textField){
		Dimension size = textField.getPreferredSize();
		size.width = textField.getMaximumSize().width;
		textField.setMaximumSize(size);
	}
	
	private FormulaTextField[] createBoundPanel(String startString, String startContent,
								 		  String endString, String endContent, 
								 		  JPanel main, int FieldSize, boolean isCount){
		
		JPanel commonPanel = new JPanel();
		commonPanel.setLayout(new BoxLayout(commonPanel, BoxLayout.Y_AXIS));
		
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		FormulaTextField startField = createVariablePanel(startString, panel, startContent, FieldSize, true, isCount);
		panel.add(Box.createHorizontalStrut(3));
		FormulaTextField endField = createVariablePanel(endString, panel, endContent, FieldSize, true, isCount);
		
		commonPanel.add(Box.createVerticalStrut(6));
		commonPanel.add(panel);

		main.add(commonPanel);
		
		return new FormulaTextField[] { startField, endField };
		
	}
	
	private class MyCaretListener implements CaretListener {
		FormulaTextField myField;

		MyCaretListener(FormulaTextField field) {
			myField = field;
		}

		public void caretUpdate(CaretEvent e) {
			myIsSaved = false;
			myField.check();
			init();
		}
	}

	
	private JPanel createFormulaPanel(){
		JPanel main = new JPanel();
		main.setLayout(new BoxLayout(main, BoxLayout.Y_AXIS));
		main.setBorder(BorderFactory.createEmptyBorder(12, 12, 12, 12));
		
		myTextXFormula = createVariablePanel("x", main, "200*sin(u)*cos(t)", 15, false, false);
		main.add(Box.createVerticalStrut(12));
		myTextYFormula = createVariablePanel("y", main, "200*sin(u)*sin(t)", 15, false, false);
		main.add(Box.createVerticalStrut(12));
		myTextZFormula = createVariablePanel("z", main, "200*cos(u)", 15, false, false);
		main.add(Box.createVerticalStrut(27));
		
		main.add(new JLabel("bounds of variables"), CENTER_ALIGNMENT);
		FormulaTextField[] textFields = createBoundPanel("u:   from", "0", "to", "2 * pi", main, 3, false);
		myTextStartU = textFields[0];
		myTextEndU = textFields[1];
		main.add(Box.createVerticalStrut(10));

		textFields = createBoundPanel("t:   from", "0", "to", "2 * pi", main, 3, false);
		myTextStartT = textFields[0];
		myTextEndT = textFields[1];
		main.add(Box.createVerticalStrut(35));
		
		textFields = createBoundPanel("count U", "40", "count T", "40", main, 3, true);
		myTextCountU = textFields[0];
		myTextCountT = textFields[1];
		main.add(Box.createVerticalStrut(35));
		
		main.add(Box.createVerticalStrut(12));
		
		JPanel flow = new JPanel(new FlowLayout(FlowLayout.RIGHT, 0, 0));
		myDrawButton = new JButton("draw");
		myDrawButton.addActionListener(new MyActionListener());
		flow.add(myDrawButton);
		main.add(flow);
		return main;
	}
	
	private class MyActionListener implements ActionListener{
		public void actionPerformed(ActionEvent e){
			new MyComponent(mySurface);	
		} 
	}
	
	private void init(){
		myDrawButton.setEnabled(myWrongFieldCounter == 0);
		if (myWrongFieldCounter == 0) {
			myFormula = new Formula(myTextXFormula.getText(),myTextYFormula.getText(),
					myTextZFormula.getText(), myTextStartT.getText(), myTextEndT.getText(),
					myTextStartU.getText(), myTextEndU.getText());
			mySurface = new Surface(myFormula, myTextCountU.getText(), myTextCountT.getText());
		}	
	}
	
	private void initAfterReading() {
		myTextXFormula.setText(myFormula.getXFormula());
		myTextXFormula.check();
		myTextYFormula.setText(myFormula.getYFormula());
		myTextYFormula.check();
		myTextZFormula.setText(myFormula.getZFormula());
		myTextZFormula.check();
		myTextStartU.setText(myFormula.getStringStartU());
		myTextStartU.check();
		myTextEndU.setText(myFormula.getStringEndU());
		myTextEndU.check();
		myTextStartT.setText(myFormula.getStringStartT());
		myTextStartT.check();
		myTextEndT.setText(myFormula.getStringEndT());
		myTextEndT.check();
		myTextCountU.setText(mySurface.getStringCountU());
		myTextCountU.check();
		myTextCountT.setText(mySurface.getStringCountT());
		myTextCountT.check();
	}
	
	public void increaseWrongFieldCounter() {
		myWrongFieldCounter++;
	}
	
	public void decreaseWrongFieldCounter() {
		myWrongFieldCounter--;
	}
}







